(define-module (ui mass-management hosts ajax)
			   :use-module (alterator ajax)
			   :use-module (alterator woo)
			   :use-module (alterator str)
			   :export (init))

(define (ui-hosts-buttons-activity active)
  (form-update-activity "import_hosts_file" active)
  (form-update-activity "import_hosts_button" active)
  (form-update-activity "export_hosts_button" active)
  (form-update-activity "add_host" active)
  (form-update-activity "add_host_button" active)
  (form-update-activity "delete_host_button" active)
  (form-update-activity "delete_group_button" active)
  (form-update-activity "add_key" active)
  (form-update-activity "host_key_file" active)
  (form-update-activity "add_key_button" active))


(define (ui-hosts-tab-activity active)
  (form-update-activity "add_group" active)
  (form-update-activity "add_group_button" active)
  (form-update-activity "grouplist" active)
  (form-update-visibility "nomaster_page" (not active))
  (form-update-activity "pull_group" active)
  (form-update-activity "pull_group_label" active)
  ; Don't enable other controls:
  ; they are will be handled in another way
  (or active
		(ui-hosts-buttons-activity #f))
  )

(define (ui-pull-group-visibility is-pull-group)
  (form-update-visibility "add_ip_section" (not is-pull-group))
  (form-update-visibility "add_key_section" is-pull-group))

(define (read-groups)
  (form-update-enum "grouplist" (woo-list "/mass-management/avail_groups")))

(define (read-hosts)
  (let* ((group (form-value "grouplist"))
		 (not-empty (not-empty-string? group)))
	(and
	  not-empty
	  (catch/message
		(lambda()
		  (form-update-enum "hostlist" (woo-list "/mass-management/avail_hosts" 'group group)))))
	(begin  (ui-hosts-buttons-activity not-empty)
			(catch/message
			  (lambda()
				(ui-pull-group-visibility (woo-get-option
											(woo-read-first "/mass-management/is_pull_group"
															'group (form-value "grouplist"))
											'pull_group))))
			)))

(define (add-group)
  (let ((name (form-value "add_group"))
	(pull (form-value "pull_group")))
	(and (catch/message
		   (lambda()
			 (woo-write "/mass-management/add_group" 'group name 'pull_group pull 'language (form-value "language"))))
		 (read-groups)
		 (form-update-value "grouplist" name)
		 (form-update-value "add_group" "")
		 (form-update-value "pull_group" #f)
		 (form-update-enum "hostlist" '())
		 (ui-hosts-buttons-activity #t)
		 (ui-pull-group-visibility pull))))

(define (del-group)
  (and (catch/message
		 (lambda()
		   (woo-write "/mass-management/del_group" 'group (form-value "grouplist") 'language (form-value "language"))))
	   (read-groups)
	   (form-update-enum "hostlist" '())
	   (ui-hosts-buttons-activity #f)))

(define (add-host)
  (and (catch/message
		 (lambda()
		   (woo-write "/mass-management/add_host" 'group (form-value "grouplist") 'add_host (form-value "add_host") 'language (form-value "language"))))
	   (read-hosts)
	   (form-update-value "add_host" "")))

(define (add-key)
  (and
	(call-with-form-file
	  "host_key_file"
	  (lambda(path)
		(catch/message
		  (lambda() (woo-write "/mass-management/add_host" 'group (form-value "grouplist") 'add_host path 'language (form-value "language"))))))
	(read-hosts)))

(define (del-host)
  (and (catch/message
		 (lambda()
		   (woo-write "/mass-management/del_host" 'group (form-value "grouplist") 'del_host (form-value "hostlist") 'language (form-value "language"))))
	   (read-hosts)))

(define (export-hosts)
  (form-replace "/mass-management/export-hosts" 'group (form-value "grouplist")))

(define (import-hosts)
  (and
	(call-with-form-file
	  "import_hosts_file"
	  (lambda(path)
		(catch/message
		  (lambda() (woo-write "/mass-management/import_hosts" 'group (form-value "grouplist") 'import_hosts_file path 'language (form-value "language"))))))
	(read-hosts)))

;;; Read mode
(define (read-mode)
  (catch/message
	(lambda()
	  (woo-get-option (woo-read-first "/mass-management/mode"
					  'language (form-value "language"))
			  'mode))))

(define (init)
  (read-groups)
  (ui-hosts-buttons-activity #f)
  (ui-hosts-tab-activity (string=? "master" (read-mode)))
  (ui-pull-group-visibility #f)
  (form-bind-upload "import_hosts_button" "click" "import_hosts_file" import-hosts)
  (form-bind-upload "add_key_button" "click" "host_key_file" add-key)
  (form-bind "export_hosts_button" "click" export-hosts)
  (form-bind "add_group_button" "click" add-group)
  (form-bind "delete_group_button" "click" del-group)
  (form-bind "grouplist" "change" read-hosts)
  (form-bind "add_host_button" "click" add-host)
  (form-bind "delete_host_button" "click" del-host))
