(define-module (ui mass-management profile ajax)
			   :use-module (alterator ajax)
			   :use-module (alterator woo)
			   :use-module (alterator str)
			   :export (init))

(define *prev-neteth-iface* "")

;; profile
(define (list-profiles)
  (form-update-enum "profilelist" (woo-list "/mass-management/avail_profiles")))

(define (add-profile)
  (let ((name (form-value "new_profile")))
	(and (catch/message
		   (lambda()
			 (woo-write "/mass-management/add_profile" 'new_profile name)))
		 (list-profiles)
		 (form-update-value "profilelist" name)
		 (form-update-value "new_profile" ""))))

(define (del-profile)
  (and (catch/message
		 (lambda()
		   (woo-write "/mass-management/del_profile" 'profile (form-value "profilelist"))))
	   (list-profiles)))

(define (read-profile)
  (let ((profile (form-value "profilelist")))
	(and (catch/message
		   (lambda()
			 (form-update-value-list (woo-read-first "/mass-management/profile"
													 'profile profile
													 'language (form-value "language")))))
		 (list-profile-routes profile)
		 ; reset neteth_iface value, it can't be valid when profile is changed
		 (form-update-value "neteth_iface" "")
		 (list-neteth-ifaces profile)
		 (read-neteth-iface profile ""))))

(define (commit-profile)
  (and (catch/message
		 (lambda()
		   (apply woo-write "/mass-management/profile" (form-value-list))))
	   (read-profile)))

(define (list-netmasks)
  (catch/message
	(lambda()
	  (let ((masks (woo-list "/net-eth/avail_masks" 'ipv 4 'language (form-value "language"))))
		(form-update-enum "neteth_mask" masks)
		(form-update-value "neteth_mask" "24")
		(form-update-enum "route_src_mask" masks)
		(form-update-value "route_src_mask" "24")))))

;; routes
(define (list-profile-routes profile)
  (catch/message
	(lambda()
	  (form-update-enum "routes_list" (woo-list "/mass-management/avail_profile_routes"
												'profile profile
												'language (form-value "language"))))))

(define (add-route)
  (let ((profile (form-value "profilelist")))
	(and (catch/message
		   (lambda()
			 (woo-write "/mass-management/add_profile_route"
						'profile profile
						'route_iface_name (form-value "route_iface_name")
						'route_src (form-value "route_src")
						'route_src_mask (form-value "route_src_mask")
						'route_dst_ip (form-value "route_dst_ip")
						'route_dst_iface (form-value "route_dst_iface")
						'route_metric (form-value "route_metric")
						'language (form-value "language"))))
		 (list-profile-routes profile))))

(define (del-route)
  (let ((profile (form-value "profilelist")))
	(and (catch/message
		   (lambda()
			 (woo-write "/mass-management/del_profile_route"
						'profile profile
						'routes_list (form-value "routes_list")
						'language (form-value "language"))))
		 (list-profile-routes profile))))

;; dhcpd
(define (list-dhcpd-leasetimes)
  (catch/message
	(lambda()
	  (form-update-enum "dhcpd_client_time" (woo-list "/dhcp/avail_time" 'language (form-value "language"))))))

;; neteth
(define (ui-new-iface visible)
  (form-update-visibility "neteth_new_iface" visible)
  (form-update-visibility "neteth_newiface_button" (not visible))
  (form-update-visibility "neteth_addiface_button" visible))

(define (list-neteth-ifaces profile)
  (catch/message
	(lambda()
	  (form-update-enum "neteth_iface" (woo-list "/mass-management/avail_neteth_ifaces"
													 'profile profile
													 'language (form-value "language"))))))

(define (list-neteth-addresses profile iface)
  (catch/message
	(lambda()
	  (form-update-enum "neteth_addresses" (woo-list "/mass-management/avail_neteth_addresses"
													 'profile profile
													 'neteth_iface iface
													 'language (form-value "language"))))))

(define (list-neteth-configurations)
  (catch/message
	(lambda()
	  (form-update-enum "neteth_iface_configuration" (woo-list "/net-eth/avail_configurations"
														 'ipv 4
														 'language (form-value "language"))))))

(define (read-neteth-iface profile iface)
  (and
	   (catch/message
		 (lambda()
		   (form-update-value-list (woo-read-first "/mass-management/neteth_iface"
												   'profile profile
												   'neteth_iface iface
												   'language (form-value "language")))))
		 ; neteth_iface value can be set from backend
		 (set! *prev-neteth-iface* (if (not-empty-string? iface)
									 iface
									 (form-value "neteth_iface")))

		 (list-neteth-addresses profile *prev-neteth-iface*)))

(define (write-neteth-iface profile iface)
  (and (not-empty-string? iface)
	   (catch/message
		 (lambda()
		   (woo-write "/mass-management/neteth_iface"
					  'profile profile
					  'neteth_iface iface
					  'neteth_iface_configuration (form-value "neteth_iface_configuration")
					  'neteth_iface_default_gw (form-value "neteth_iface_default_gw")
					  'neteth_iface_dns (form-value "neteth_iface_dns")
					  'neteth_iface_search (form-value "neteth_iface_search")
					  'language (form-value "language"))))))

(define (ui-change-neteth-iface)
  (let ((profile (form-value "profilelist"))
		(name (form-value "neteth_iface")))
	(and (not-empty-string? *prev-neteth-iface*)
		 (write-neteth-iface profile *prev-neteth-iface*))
	(and (not-empty-string? name)
		 (read-neteth-iface profile name))))

(define (new-neteth-iface)
  (ui-new-iface #t))

(define (add-neteth-iface)
  (let ((profile (form-value "profilelist"))
		(name (form-value "neteth_new_iface")))
	(and (catch/message
		   (lambda()
			 (woo-write "/mass-management/add_neteth_iface"
						'profile profile
						'neteth_iface name
						'language (form-value "language"))))
		 (list-neteth-ifaces profile)
		 (form-update-value "neteth_iface" name)
		 (form-update-value "neteth_new_iface" "")
		 (form-update-value "neteth_addresses" "")
		 (ui-new-iface #f)
		 (ui-change-neteth-iface))))

(define (del-neteth-iface)
  (let ((profile (form-value "profilelist"))
		(name (form-value "neteth_iface")))
	(and (not-empty-string? name)
		 (catch/message
		   (lambda()
			 (woo-write "/mass-management/del_neteth_iface"
						'profile profile
						'neteth_iface name
						'language (form-value "language"))))
		 (list-neteth-ifaces profile)
		 (read-neteth-iface profile ""))))

(define (add-neteth-ip)
  (let ((profile (form-value "profilelist"))
		(iface (form-value "neteth_iface")))
	(and (catch/message
		   (lambda()
			 (woo-write "/mass-management/add_neteth_ip"
						'profile profile
						'neteth_iface iface
						'neteth_addip (form-value "neteth_addip")
						'neteth_mask (form-value "neteth_mask")
						'language (form-value "language"))))
		 (list-neteth-addresses profile iface))))

(define (del-neteth-ip)
  (let ((profile (form-value "profilelist"))
		(iface (form-value "neteth_iface")))
	(and (catch/message
		   (lambda()
			 (woo-write "/mass-management/del_neteth_ip"
						'profile profile
						'neteth_iface iface
						'neteth_delip (form-value "neteth_addresses")
						'language (form-value "language"))))
		 (list-neteth-addresses profile iface))))

;; firewall
(define (firewall-list-tables)
  (catch/message
	(lambda()
	  (form-update-enum "firewall_tables" (woo-list "/net-iptables-manual" 'language (form-value "language"))))))

(define (read-firewall-rules)
  (catch/message
	(lambda()
	  (form-update-value-list (woo-read-first "/mass-management/firewall_rules" 'profile (form-value "profilelist")
				'firewall_table (form-value "firewall_tables"))))))


(define (write-firewall-rules)
  (and
	(form-value "firewall_change")
	(catch/message
	  (lambda()
		(woo-write "/mass-management/firewall_rules" 'profile (form-value "profilelist")
				   'firewall_table (form-value "firewall_tables")
				   'firewall_rules_text (form-value "firewall_rules_text"))))))

(define (clear-firewall-rules)
  (form-update-value "firewall_rules_text" ""))

(define (export-firewall)
  (form-replace "/mass-management/export-firewall"
				'profile (form-value "profilelist")
				'table (form-value "firewall_tables")))

(define (import-firewall)
  (and
	(call-with-form-file
	  "import_firewall_rules"
	  (lambda(path)
		(catch/message
		  (lambda() (woo-write "/mass-management/import_firewall"
							   'profile (form-value "profilelist")
							   'table (form-value "firewall_tables")
							   'import_firewall_rules path
							   'language (form-value "language"))))))
	(read-firewall-rules)))

;;
(define (init)
  (list-profiles)
  (list-netmasks)
  (list-dhcpd-leasetimes)
  (ui-new-iface #f)
  (list-neteth-configurations)
  (list-neteth-ifaces (form-value "profilelist"))
  (ui-change-neteth-iface)
  (firewall-list-tables)
  (form-bind "add_profile_button" "click" add-profile)
  (form-bind "delete_profile_button" "click" del-profile)
  (form-bind "profilelist" "change" read-profile)
  (form-bind "add_route_to_list_button" "click" add-route)
  (form-bind "del_route_to_list_button" "click" del-route)
  (form-bind "neteth_newiface_button" "click" new-neteth-iface)
  (form-bind "neteth_addiface_button" "click" add-neteth-iface)
  (form-bind "neteth_deliface_button" "click" del-neteth-iface)
  (form-bind "neteth_iface" "change" ui-change-neteth-iface)
  (form-bind "neteth_addip_button" "click" add-neteth-ip)
  (form-bind "neteth_delip_button" "click" del-neteth-ip)
  (form-bind "firewall_tables" "change" read-firewall-rules)
  (form-bind "firewall_save_button" "click" write-firewall-rules)
  (form-bind "firewall_clear_button" "click" clear-firewall-rules)
  (form-bind-upload "import_firewall_button" "click" "import_firewall_rules" import-firewall)
  (form-bind "export_firewall_button" "click" export-firewall)
  (form-bind "apply_profile_button" "click" commit-profile))
